<?php

namespace RosreestrGeomToWKT;

use RosreestrGeomToWKT\Ordinate;
use RosreestrGeomToWKT\SpelementUnit;
use RosreestrGeomToWKT\SpatialElement;
use RosreestrGeomToWKT\EntitySpatial;
use RosreestrGeomToWKT\Contours;
use RosreestrGeomToWKT\WktUtilites;

class Processor 
{
    /**
    * @var object
    */
    private $ut;

    /**
    * @var array
    */
    private $xmlNode;

    /**
    * @var bool
    */
    private $ifContours;

    /**
    * @param array $xmlNode
    */
    public function __construct(array $xmlNode, bool $ifContours)
    {
        $this->xmlNode = $xmlNode;
        $this->ifContours = $ifContours;
        $this->ut = new WktUtilites();
    }

    /**
    * @return string
    */
    public function getWKT(): string
    {
        try 
        {
            if ($this->ifContours) {
                return $this->parseContoursNode($this->xmlNode);
            } else {
                return $this->parseEntitySpatialNode($this->xmlNode);
            }
        }
        catch (Exception $e)
        {
            return ['success' => false, 'error' => $e->getMessage()];
        }
    }

    /**
    * @param array
    * @return string
    */
    private function parseContoursNode(array $contoursNode): string
    {
        if ($this->ut::validateArray($contoursNode)) {
            if (array_key_exists('name', $contoursNode)) {
                if ($contoursNode['name'] === 'Contours') {
                    if (array_key_exists('value', $contoursNode)) {
                        $contoursData = $contoursNode['value'];
                        if ($this->ut::validateArray($contoursData['value'])) {
                            $entitySpatials = [];
                            foreach ($contoursData['value'] as $contour) {
                                if (array_key_exists('value', $contour)) {
                                    if ($this->ut::validateArray($contour['value'])) {
                                        if (count($contour['value']) === 1) {
                                            try {
                                                $entitySpatial = parseEntitySpatialNode($contour['value']);
                                                $entitySpatials[] = $entitySpatial;
                                            } catch (Exception $e) {
                                                throw new \Exception($e->getMessage());
                                            }
                                        } else {
                                            throw new \Exception('Incorrect count of counter inner elements');
                                        }
                                    } else {
                                        throw new \Exception('Invalid node structure. Attribute "value" is not found');
                                    }
                                } else {
                                    throw new \Exception('Invalid node structure. Attribute "value" is not found');
                                }
                            }
                            if ($this->ut::validateArray($entitySpatials)) {
                                try {
                                    $coutObject = new Contours($entitySpatials);
                                    return $countObject->contoursData;
                                } catch (Exception $e) {
                                    throw new \Exception($e->getMessage());
                                }
                            } else {
                                throw new \Exception('Incorrect count of counter inner elements');
                            }
                        } else {
                            throw new \Exception('Contours not found');
                        }
                    } else {
                        throw new \Exception('Invalid node structure. Attribute "value" is not found');
                    }
                } else {
                    throw new \Exception('Node name is incorrect');
                }
            } else {
                throw new \Exception('Invalid node structure. Attribute "name" is not found');
            }
        } else {
            throw new \Exception('Invalid Contours node structure. Node is not an array, or empty array');
        }
    }

    /**
    * @param array
    * @return string
    */
    private function parseEntitySpatialNode(array $entitySpatialNode)
    {
        if ($this->ut::validateArray($entitySpatialNode)) {
            if (array_key_exists('name', $entitySpatialNode)) {
                $nodeName = $entitySpatialNode['name'];
                if ($nodeName && $nodeName === 'EntitySpatial') {
                    if (array_key_exists('value', $entitySpatialNode)) {
                        $spatialElementsData = $entitySpatialNode['value'];
                        if ($this->ut::validateArray($spatialElementsData)) {
                            $spatialElements = [];
                            foreach ($spatialElementsData as $spatialElement) {
                                if (array_key_exists('value', $spatialElement)) {
                                    $spelementUnitsData = $spatialElement['value'];
                                    if ($this->ut::validateArray($spelementUnitsData)) {
                                        $spelementUnits = [];
                                        foreach ($spelementUnitsData as $spelementUnit) {
                                            $typeUnit = "";
                                            $suNmb = "";
                                            if (array_key_exists('attributes', $spelementUnit)) {
                                                if ($this->ut::validateArray($spelementUnit['attributes'])) {
                                                    $typeUnit = array_key_exists('TypeUnit', $spelementUnit['attributes']) ? $spelementUnit['attributes']['TypeUnit'] : '';
                                                    $suNmb = array_key_exists('SuNmb', $spelementUnit['attributes']) ? $spelementUnit['attributes']['SuNmb'] : '';
                                                }
                                            }
                                            if (array_key_exists('value', $spelementUnit)) {
                                                if ($this->ut::validateArray($spelementUnit['value'])) {
                                                    $ordinateData = $spelementUnit['value'][0];
                                                    if (array_key_exists('attributes', $ordinateData)) {
                                                        $x = array_key_exists('X', $ordinateData['attributes']) ? $ordinateData['attributes']['X'] : '';
                                                        $y = array_key_exists('Y', $ordinateData['attributes']) ? $ordinateData['attributes']['Y'] : '';
                                                        $ordNmb = array_key_exists('OrdNmb', $ordinateData['attributes']) ? $ordinateData['attributes']['OrdNmb'] : '';
                                                        $deltaGeopoint = array_key_exists('DeltaGeopoint', $ordinateData['attributes']) ? $ordinateData['attributes']['DeltaGeopoint'] : '';

                                                        try {
                                                            $ordinate = new Ordinate($x, $y, $ordNmb, $deltaGeopoint);
                                                            $spelUnit = new SpelementUnit($typeUnit, $suNmb, $ordinate);
    
                                                            $spelementUnits[] = $spelUnit;
                                                        } catch (Exception $e) {
                                                            break 2;
                                                            throw new \Exception($e->getMessage());
                                                        }
                                                    } else {
                                                        break 2;
                                                        throw new \Exception('Ordinate not found');
                                                    }
                                                } else {
                                                    break;
                                                    throw new \Exception('Invalid spelement unit');
                                                }
                                            } else {
                                                break;
                                                throw new \Exception('Invalid spelement unit');
                                            }
                                        }
                                        if ($this->ut::validateArray($spelementUnits)) {
                                            try {
                                                $spatEl  = new SpatialElement($spelementUnits);
                                                $spatialElements[] = $spatEl;
                                            } catch (Exception $e) {
                                                break;
                                                throw new \Exception($e->getMessage());
                                            }
                                        } else {
                                            break;
                                            throw new \Exception('Invalid spelement unit');
                                        }
                                    } else {
                                        break;
                                        throw new \Exception('Invalid spelement unit');
                                    }
                                } else {
                                    break;
                                    throw new \Exception('Invalid spatial element');
                                }
                            }
                            if ($this->ut::validateArray($spatialElements)) {
                                try {
                                    $entitySp = new EntitySpatial($spatialElements);
                                    if ($this->ifContours) {
                                        return $entitySp;
                                    } else {
                                        return $entitySp->entitySpatialData['type'] . $entitySp->entitySpatialData['coordinates'];
                                    }
                                } catch (Exception $e) {
                                    throw new \Exception($e->getMessage());
                                }
                            }
                        } else {
                            throw new \Exception('SpatialElements not found');
                        }
                    } else {
                        throw new \Exception('Invalid node structure. Attribute "value" is not found');
                    }
                } else {
                    throw new \Exception('Node name is incorrect');
                }
            } else {
                throw new \Exception('Invalid node structure. Attribute "name" is not found');
            }
        } else {
            throw new \Exception('Invalid EntitySpatial node structure. The node is not an array, or empty array');
        }
    }
}
